package com.ali.notificationsvc.config;

import com.ali.notificationsvc.deserialization.NotificationDeserializer;
import com.ali.notificationsvc.dtos.CreateNotificationDto;
import org.apache.kafka.clients.CommonClientConfigs;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.common.config.SaslConfigs;
import org.apache.kafka.common.serialization.LongDeserializer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerContainerFactory;
import org.springframework.kafka.core.ConsumerFactory;
import org.springframework.kafka.core.DefaultKafkaConsumerFactory;
import org.springframework.kafka.listener.ConcurrentMessageListenerContainer;
import org.springframework.kafka.listener.ContainerProperties;
import org.springframework.kafka.listener.DefaultErrorHandler;
import org.springframework.retry.backoff.ExponentialBackOffPolicy;
import org.springframework.retry.policy.SimpleRetryPolicy;
import org.springframework.retry.support.RetryTemplate;
import org.springframework.util.backoff.FixedBackOff;

import java.util.HashMap;
import java.util.Map;

@Configuration
public class NotificationKafkaConsumer {
    @Value("${spring.kafka.bootstrap-servers}")
    private String bootstrapServers;

    @Value("${spring.kafka.properties.security.protocol}")
    private String securityProtocol;

    @Value("${spring.kafka.properties.sasl.mechanism}")
    private String saslMechanism;

    @Value("${spring.kafka.properties.sasl.jaas.config}")
    private String saslJaasConfig;

    @Bean
    public Map<String, Object> notificationConsumerConfig(){
        Map<String,Object> props = new HashMap<>();
        props.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG,bootstrapServers);
        props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, LongDeserializer.class);
        props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, NotificationDeserializer.class);
        props.put(CommonClientConfigs.SECURITY_PROTOCOL_CONFIG, securityProtocol);
        props.put(SaslConfigs.SASL_MECHANISM, saslMechanism);
        props.put(SaslConfigs.SASL_JAAS_CONFIG, saslJaasConfig);
        return props;
    }

    @Bean
    public ConsumerFactory<Long, CreateNotificationDto> notificationConsumerFactory(){
        return new DefaultKafkaConsumerFactory<>(notificationConsumerConfig());
    }

    @Bean
    public KafkaListenerContainerFactory<ConcurrentMessageListenerContainer<Long,CreateNotificationDto>> notificationFactory
            (ConsumerFactory<Long,CreateNotificationDto> notificationConsumerFactory){
        ConcurrentKafkaListenerContainerFactory<Long,CreateNotificationDto> notificationFactory
                = new ConcurrentKafkaListenerContainerFactory<>();
        notificationFactory.setConsumerFactory(notificationConsumerFactory);
        // Set acknowledgment mode to manual
        notificationFactory.getContainerProperties().setAckMode(ContainerProperties.AckMode.MANUAL);

        // Set the error handler with retry functionality
        notificationFactory.setCommonErrorHandler(new DefaultErrorHandler(
                new FixedBackOff(1000L, 5)));
        return notificationFactory;
    }

}
